@@ -10,6 +10,11 @@ from plone.memoize.instance import memoize
 from Products.statusmessages.interfaces import IStatusMessage
 
 
+DEFAULT_PORTRAIT_URL = "defaultUser.gif"
+
+
+def cmpMemberDict(a, b):
+	return cmp(a["fullname"], b["fullname"])
 
 
 class MemberList(BrowserView):
@@ -28,30 +33,62 @@ class MemberList(BrowserView):
 			member = pm.getMemberById(memberId)
 			if member:
 				out.append(self._memberInfo(memberId, member))
+		out.sort(cmpMemberDict)
 		return out
 
-	def previousMembers(self):
+	def oldMembers(self):
 		pm = self.ctx.portal_membership
 		out = []
 		for memberId in pm.listMemberIds():
 			if not memberId in self.exclude and not memberId in self.current:
 				member = pm.getMemberById(memberId)
 				out.append(self._memberInfo(memberId, member))
+
+		for u in self.ctx.getNonusers():
+			x = u.split(":", 1)
+			name = x[0]
+			if len(x) == 2:
+				description = x[1]
+			else:
+				description = None
+
+			out.append(dict(
+				username = None,
+				email = None,
+				homeFolder = None,
+				fullname = name,
+				portrait_url = DEFAULT_PORTRAIT_URL,
+				description = description,
+				homepage = None))
+
+		out.sort(cmpMemberDict)
 		return out
 
 	def _memberInfo(self, memberId, member):
 		portal = getToolByName(self.ctx, "portal_url").getPortalObject()
-		portrait = portal.portal_memberdata.portraits.get("espeak")
+		portrait = portal.portal_memberdata.portraits.get(memberId)
 		if portrait:
-			portrait = portrait.absolute_url()
+			portrait_url = portrait.absolute_url()
 		else:
-			portrait = "defaultUser.gif"
-		homeFolder = "%s/%s" % (portal.Members.absolute_url(), memberId)
+			portrait_url = DEFAULT_PORTRAIT_URL
+
+		homeFolder = portal.Members.get(memberId)
+		if homeFolder and len(homeFolder.contentItems()) > 0:
+			homeFolder = homeFolder.absolute_url()
+		else:
+			homeFolder = None
+
+		if self.ctx.portal_membership.isAnonymousUser():
+			email = member.getProperty("email").replace("@", " (at) ")
+		else:
+			email = member.getProperty("email")
 
 		return dict(
-				email = member.getProperty("email"),
+				username = memberId,
+				email = email,
 				fullname = member.getProperty("fullname"),
-				portrait_url = member.getPersonalPortrait().absolute_url(),
+				portrait_url = portrait_url,
 				homeFolder = homeFolder,
-				description = member.getProperty("description")
+				description = member.getProperty("description"),
+				homepage = member.getProperty("home_page")
 			)
